import type { ItemDetails, Overview } from "../types"
import type { Crypto } from "./crypto"
import { Attachment } from "./attachment"

export interface EncryptedItem {
  category: string // "001"
  /** Unix seconds */
  created: integer
  d: string // "b3BkYXRhMbt"
  folder: string // 32 chars
  hmac: string // base64
  k: string // base64
  o: string // base64
  tx: integer // Unix seconds
  updated: integer // Unix seconds
  uuid: string // 32 chars
}

export class Item {
  #crypto: Crypto
  #data: EncryptedItem

  attachments: Attachment[] = []

  get uuid() {
    return this.#data.uuid
  }
  get overview(): Overview {
    return this.#crypto.decryptItemOverview(this.#data)
  }
  get itemDetails(): ItemDetails {
    return this.#crypto.decryptItemDetails(this.#data)
  }

  constructor(crypto: Crypto, data: EncryptedItem) {
    this.#crypto = crypto
    this.#data = data
  }

  /** @internal */
  addAttachment(buffer: Buffer) {
    this.attachments.push(new Attachment(this.#crypto, this.#data.k, buffer))
  }
}
